<?php
namespace BancoTalentos\Util;

use BancoTalentos\Models\ServiceUser;
use GuzzleHttp\Client;

class GoogleAuth {
    public const URL = 'https://www.googleapis.com/oauth2/v3/tokeninfo';

    /** @var string */
    protected $clientId;

    /** @var Client */
    protected $http;

    public function __construct(
        string $clientId
    ) {
        $this->http = new Client();
        $this->clientId = $clientId;
    }

    public function fetchUser(
        string $idToken
    ): ServiceUser {
        $tokenData = $this->fetchDataFromIdToken($idToken);
        $this->verifyTokenData($tokenData);

        $id = (string) $tokenData['sub'];
        $name     = (string) $tokenData['name'];
        $email    = (string) $tokenData['email'];
        $pictureUrl = $tokenData['picture'];

        $user = new ServiceUser($id, $name, $email, $pictureUrl);

        return $user;
    }

    public function fetchDataFromIdToken(
        string $idToken
    ): array {
        $params = [
            'id_token' => $idToken,
        ];

        $response = $this->http->request('POST', self::URL, [
            'json' => $params,
        ]);

        $content = $response->getBody()->getContents();
        $body = json_decode($content, true);

        return $body;
    }

    public function verifyTokenData(
        array $tokenData
    ): bool {
        $this->validateTokenError($tokenData);
        $this->validateTokenAud($tokenData);
        $this->validateTokenIss($tokenData);
        $this->validateTokenExp($tokenData);

        return true;
    }

    private function validateTokenError(
        array $tokenData
    ): void {
        if (!empty($tokenData['error'])) {
            throw new \Exception($tokenData['error']);
        }
    }

    private function validateTokenAud(
        array $tokenData
    ): void {
        $sameClientId = $tokenData['aud'] === $this->clientId;

        if (!$sameClientId) {
            throw new \Exception('Client ID do token é inválido!');
        }
    }

    private function validateTokenIss(
        array $tokenData
    ): void {
        $iss = $tokenData['iss'];

        $validIss =
            $iss === 'accounts.google.com' ||
            $iss === 'https://accounts.google.com';

        if (!$validIss) {
            throw new \Exception('Issuer do token é inválido!');
        }
    }

    private function validateTokenExp(
        array $tokenData
    ): void {
        $expirationTime = $tokenData['exp'];
        $currentTime = time();

        $expired = $expirationTime < $currentTime;

        if ($expired) {
            throw new \Exception('O token expirou!');
        }
    }

}