<?php
namespace BancoTalentos\Util;

use BancoTalentos\Models\ServiceUser;
use GuzzleHttp\Client;

class LinkedInAuth {
    public const URL_ACCESS_TOKEN_TEMPLATE = 'https://www.linkedin.com/oauth/v2/accessToken?grant_type=authorization_code&code={authorization_code}&redirect_uri={callback_url}&client_id={client_id}&client_secret={client_secret}';

    public const URL_PROFILE = 'https://api.linkedin.com/v2/me?projection=(id,firstName,lastName,profilePicture(displayImage~:playableStreams))';

    public const URL_EMAIL = 'https://api.linkedin.com/v2/emailAddress?q=members&projection=(elements*(handle~))';

    public const SYSTEM_LOCALE = 'pt_BR';

    /** @var string */
    protected $clientId;

    /** @var string */
    protected $clientSecret;

    /** @var Client */
    protected $http;

    public function __construct(
        string $clientId,
        string $clientSecret
    ) {
        $this->http = new Client();
        $this->clientId = $clientId;
        $this->clientSecret = $clientSecret;
    }

    public function fetchUser(
        string $authorizationCode,
        string $callbackUrl
    ): ServiceUser {
        $accessToken = $this->fetchAccessToken($authorizationCode, $callbackUrl);

        $email = $this->fetchEmail($accessToken);
        $profile = $this->fetchProfile($accessToken);

        $firstName = $this->getFirstName($profile);
        $lastName = $this->getLastName($profile);
        $name = "$firstName $lastName";

        $id = $profile['id'];

        $pictureUrl = $this->getPictureUrl($profile);

        $user = new ServiceUser($id, $name, $email, $pictureUrl);

        return $user;
    }

    public function fetchAccessToken(
        string $authorizationCode,
        string $callbackUrl
    ): string {
        $vars = [
            '{authorization_code}'  => $authorizationCode,
            '{callback_url}'        => $callbackUrl,
            '{client_id}'           => $this->clientId,
            '{client_secret}'       => $this->clientSecret,
        ];

        $accessTokenUrl = strtr(self::URL_ACCESS_TOKEN_TEMPLATE, $vars);

        $headers = [
            'Content-Type' => 'x-www-form-urlencoded',
        ];

        $response = $this->http->request('POST', $accessTokenUrl, [
            'headers' => $headers,
        ]);

        $content = $response->getBody()->getContents();
        $body = json_decode($content, true);

        $this->accessToken = $body['access_token'];

        return $this->accessToken;
    }

    public function fetchEmail(
        string $accessToken
    ): string {
        $this->verifyAccessToken($accessToken);

        $headers = [
            'Authorization' => "Bearer {$accessToken}",
        ];

        $response = $this->http->request('GET', self::URL_EMAIL, [
            'headers' => $headers,
        ]);

        $content = $response->getBody()->getContents();
        $body = json_decode($content, true);

        $email = $body['elements'][0]['handle~']['emailAddress'];

        return $email;
    }

    public function fetchProfile(
        string $accessToken
    ): array {
        $this->verifyAccessToken($accessToken);

        $headers = [
            'Authorization' => "Bearer {$accessToken}",
        ];

        $response = $this->http->request('GET', self::URL_PROFILE, [
            'headers' => $headers,
        ]);

        $content = $response->getBody()->getContents();
        $body = json_decode($content, true);

        return $body;
    }

    private function verifyAccessToken(
        string $accessToken
    ): void {
        if (empty($accessToken)) {
            throw new ExpectedException('O Access Token do LinkedIn não foi definido!');
        }
    }

    private function getFirstName(
        array $profile
    ): string {
        $firstNameData = $profile['firstName'];
        $userPreferredLocale = $firstNameData['preferredLocale'];
        $localizedName = $firstNameData['localized'];

        $userCountry = $userPreferredLocale['country'];
        $userLanguage = $userPreferredLocale['language'];

        $userLocale = "{$userLanguage}_{$userCountry}";

        $firstName = $localizedName[self::SYSTEM_LOCALE] ?? $localizedName[$userLocale] ?? null;

        if (empty($firstName)) {
            $firstNameDataJson = json_encode($firstNameData);

            throw new ExpectedException(
                'Não foi possível acessar o primeiro nome do usuário! ' .
                $firstNameDataJson
            );
        }

        return $firstName;
    }

    private function getLastName(
        array $profile
    ): string {
        $lastNameData = $profile['lastName'];
        $userPreferredLocale = $lastNameData['preferredLocale'];
        $localizedName = $lastNameData['localized'];

        $userCountry = $userPreferredLocale['country'];
        $userLanguage = $userPreferredLocale['language'];

        $userLocale = "{$userLanguage}_{$userCountry}";

        $lastName = $localizedName[self::SYSTEM_LOCALE] ?? $localizedName[$userLocale] ?? null;

        if (empty($lastName)) {
            $lastNameDataJson = json_encode($lastNameData);

            throw new ExpectedException(
                'Não foi possível acessar o sobrenome do usuário! ' .
                $lastNameDataJson
            );
        }

        return $lastName;
    }

    private function getPictureUrl(
        array $profile
    ): string {
        $pictureUrl = $profile['profilePicture']['displayImage~']['elements'][0]['identifiers'][0]['identifier'] ?? '';

        return $pictureUrl;
    }
}