<?php

namespace tests\BancoTalentos;

use PHPUnit\Framework\TestCase;

use BancoTalentos\Util\{ DoctrineEntityUtil, Repositories };
use tests\mocks\{ BTParametersAnyOrgMock, BTParametersDefaultOrgMock, EntityManagerMock };
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class DoctrineEntityUtilTest extends TestCase {

    const FLUSHING_CONFIG = [
        'PERSIST'            => true,
        'FLUSH'              => true,
        'PENDING'            => false,
    ];

    const PERSISTING_ONLY_CONFIG = [
        'PERSIST'            => true,
        'FLUSH'              => false,
        'PENDING'            => false,
    ];

    const PENDING_CONFIG = [
        'PERSIST'            => false,
        'FLUSH'              => false,
        'PENDING'            => true,
    ];

    const INSTANTIATING_ONLY_CONFIG = [
        'PERSIST'            => false,
        'FLUSH'              => false,
        'PENDING'            => false,
    ];

    public function setUp() {
        $this->entityManagerMock = new EntityManagerMock();
        $this->btParametersMock = new BTParametersDefaultOrgMock();

        InstanceManager::getInstance()->registerService('btparameters', $this->btParametersMock);
        InstanceManager::getInstance()->registerService('entitymanager', $this->entityManagerMock);
    }

    public function testCreatingNewEntity() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $entityCreatedByUtil = $doctrineEntityUtil->setNewEntity(
            $repositoryName,
            [],
            self::INSTANTIATING_ONLY_CONFIG
        );

        $this->assertTrue($entityCreatedByUtil instanceof $repositoryName);
    }

    public function testDefiningNewEntityProperties() {
        $doctrineEntityUtil = new DoctrineEntityUtil();

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
            'NMOPERADOR' => 'Default Operator',
        ];

        $entityCreatedByUtil = $doctrineEntityUtil->setNewEntity(
            Repositories::OPERADOR,
            $data,
            self::INSTANTIATING_ONLY_CONFIG
        );

        $dataSetInEntity = [
            'NRORG' => $entityCreatedByUtil->getNrorg(),
            'CDOPERADOR' => $entityCreatedByUtil->getCdoperador(),
            'NMOPERADOR' => $entityCreatedByUtil->getNmoperador(),
        ];

        $this->assertEquals($data, $dataSetInEntity);
    }

    public function testNewEntityFlush() {
        $doctrineEntityUtil = new DoctrineEntityUtil();

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
        ];

        $this->entityManagerMock->resetFlushed();

        $entityCreatedByUtil = $doctrineEntityUtil->setNewEntity(
            Repositories::OPERADOR,
            $data,
            self::FLUSHING_CONFIG
        );

        $this->assertTrue($this->entityManagerMock->isFlushed());
    }

    public function testNewEntityPersistOnly() {
        $doctrineEntityUtil = new DoctrineEntityUtil();

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetPersisted();

        $entityCreatedByUtil = $doctrineEntityUtil->setNewEntity(
            Repositories::OPERADOR,
            $data,
            self::PERSISTING_ONLY_CONFIG
        );

        $persistedButNotFlushed =
            !$this->entityManagerMock->isFlushed() &&
            $this->entityManagerMock->isPersisted();

        $this->assertTrue($persistedButNotFlushed);
    }

    public function testNewEntityPersistPending() {
        $doctrineEntityUtil = new DoctrineEntityUtil();

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetPersisted();

        $entityCreatedByUtil = $doctrineEntityUtil->setNewEntity(
            Repositories::OPERADOR,
            $data,
            self::PENDING_CONFIG
        );

        $entitiesOnPersist = $doctrineEntityUtil->getPersistPending();
        $entityOnPersist = $entitiesOnPersist[0] ?? null;

        $notFlushedOrPersisted =
            !$this->entityManagerMock->isFlushed() &&
            !$this->entityManagerMock->isPersisted();

        $this->assertEquals($entityCreatedByUtil, $entityOnPersist);
        $this->assertTrue($notFlushedOrPersisted);
    }

    public function testUpdateEntityFlush() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();

        $entityUpdatedByUtil = $doctrineEntityUtil->updateEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::FLUSHING_CONFIG
        );

        $existingEntityUpdated =
            $existingData->getCdoperador() == $entityUpdatedByUtil->getCdoperador() &&
            $existingData->getNmoperador() == $entityUpdatedByUtil->getNmoperador() &&
            $this->entityManagerMock->isFlushed();

        $this->assertTrue($existingEntityUpdated);
    }

    public function testUpdateEntityPersistOnly() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetPersisted();

        $entityUpdatedByUtil = $doctrineEntityUtil->updateEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::PERSISTING_ONLY_CONFIG
        );

        $existingEntityUpdated =
            $existingData->getCdoperador() == $entityUpdatedByUtil->getCdoperador() &&
            $existingData->getNmoperador() == $entityUpdatedByUtil->getNmoperador() &&
            !$this->entityManagerMock->isFlushed() &&
            $this->entityManagerMock->isPersisted();

        $this->assertTrue($existingEntityUpdated);
    }

    public function testUpdateEntityPersistPending() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetPersisted();

        $entityUpdatedByUtil = $doctrineEntityUtil->updateEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::PENDING_CONFIG
        );

        $entitiesOnPersist = $doctrineEntityUtil->getPersistPending();
        $entityOnPersist = $entitiesOnPersist[0] ?? null;

        $existingEntityUpdated =
            $existingData->getCdoperador() == $entityUpdatedByUtil->getCdoperador() &&
            $existingData->getNmoperador() == $entityUpdatedByUtil->getNmoperador() &&
            !$this->entityManagerMock->isFlushed() &&
            !$this->entityManagerMock->isPersisted();

        $this->assertEquals($entityUpdatedByUtil, $entityOnPersist);
        $this->assertTrue($existingEntityUpdated);
    }

    public function testCreatingNewEntityOnUpdate() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 10,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();

        $entityCreatedByUtil = $doctrineEntityUtil->updateEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::FLUSHING_CONFIG
        );

        $newEntityCreated =
            $existingData->getCdoperador() != $entityCreatedByUtil->getCdoperador() &&
            $data['NMOPERADOR'] == $entityCreatedByUtil->getNmoperador() &&
            $this->entityManagerMock->isFlushed();

        $this->assertTrue($newEntityCreated);
    }

    public function testRemoveEntityFlush() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetRemoved();

        $removedEntity = $doctrineEntityUtil->removeEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::FLUSHING_CONFIG
        );

        $existingEntityRemoved =
            $this->entityManagerMock->isRemoved() &&
            $this->entityManagerMock->isFlushed();

        $this->assertTrue($existingEntityRemoved);
    }

    public function testRemoveEntityPersistOnly() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 1,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetRemoved();

        $removedEntity = $doctrineEntityUtil->removeEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::PERSISTING_ONLY_CONFIG
        );

        $existingEntityRemoved =
            $this->entityManagerMock->isRemoved() &&
            !$this->entityManagerMock->isFlushed();

        $this->assertTrue($existingEntityRemoved);
    }

    public function testRemoveNonExistingEntity() {
        $doctrineEntityUtil = new DoctrineEntityUtil();
        $repositoryName = Repositories::OPERADOR;

        $existingData = new $repositoryName;
        $existingData->setNrorg(20);
        $existingData->setCdoperador(1);
        $existingData->setNmoperador('Default Operator');

        $this->entityManagerMock->setExistingDataForTests([$existingData]);

        $data = [
            'NRORG' => 20,
            'CDOPERADOR' => 2,
            'NMOPERADOR' => 'Default Operator 2',
        ];

        $entityKeyNames = [
            'NRORG',
            'CDOPERADOR',
        ];

        $this->entityManagerMock->resetFlushed();
        $this->entityManagerMock->resetRemoved();

        $removedEntity = $doctrineEntityUtil->removeEntity(
            $repositoryName,
            $entityKeyNames,
            $data,
            self::FLUSHING_CONFIG
        );

        $entityNotRemoved =
            $removedEntity === false &&
            !$this->entityManagerMock->isFlushed() &&
            !$this->entityManagerMock->isRemoved();

        $this->assertTrue($entityNotRemoved);
    }
}